"""
Classifies: CHEBI:138261 very long-chain fatty acyl-CoA(4-)
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_very_long_chain_fatty_acyl_CoA_4__(smiles: str):
    """
    Determines if a molecule is a very long-chain fatty acyl-CoA(4-).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a very long-chain fatty acyl-CoA(4-), False otherwise 
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for CoA structure components
    if not all(substructure in smiles for substructure in ['COP([O-])', 'OP([O-])', 'n1cnc2c(N)ncnc12']):
        return False, "Missing characteristic CoA structure components"

    # Check for 4 negative charges (phosphate deprotonation)
    charge = Chem.GetFormalCharge(mol)
    if charge != -4:
        return False, f"Total charge is {charge}, not -4"

    # Check for thioester linkage
    thioester_pattern = Chem.MolFromSmarts('C(=O)SC')
    if not mol.HasSubstructMatch(thioester_pattern):
        return False, "Missing thioester linkage"

    # Count carbons in the fatty acid chain
    fatty_acid_pattern = Chem.MolFromSmarts('CCCCCCCCCCCCCCCCCCCCCC(=O)SC')  # Pattern for C22 or longer
    if not mol.HasSubstructMatch(fatty_acid_pattern):
        return False, "Fatty acid chain length is less than 22 carbons"

    # Check for steroid-like structures (which should be excluded)
    steroid_pattern = Chem.MolFromSmarts('C1CCC2C(C1)CCC1C3CCCC3CCC12')
    if mol.HasSubstructMatch(steroid_pattern):
        return False, "Contains steroid-like structure"

    # Count carbons in the main chain
    def count_chain_carbons(mol, start_idx):
        visited = set()
        stack = [(start_idx, 0)]
        max_length = 0
        
        while stack:
            current_idx, length = stack.pop()
            if current_idx in visited:
                continue
                
            visited.add(current_idx)
            current_atom = mol.GetAtomWithIdx(current_idx)
            
            if current_atom.GetSymbol() == 'C':
                max_length = max(max_length, length)
                
                for neighbor in current_atom.GetNeighbors():
                    if neighbor.GetSymbol() == 'C' and neighbor.GetIdx() not in visited:
                        if not any(pattern in Chem.MolToSmiles(Chem.PathToSubmol(mol, [neighbor.GetIdx()])) 
                                 for pattern in ['n1cnc', 'OP', 'NC(=O)']):
                            stack.append((neighbor.GetIdx(), length + 1))
                            
        return max_length

    # Find the thioester carbon and count chain length
    thioester_matches = mol.GetSubstructMatches(thioester_pattern)
    if not thioester_matches:
        return False, "Could not identify thioester carbon"
        
    chain_length = count_chain_carbons(mol, thioester_matches[0][0])
    
    if chain_length >= 28:  # Only classify as true if chain length is 28 or more
        return True, f"Very long-chain fatty acyl-CoA(4-) with approximately {chain_length} carbons in chain"
    else:
        return False, f"Fatty acid chain length ({chain_length}) is less than 28 carbons"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:138261',
                          'name': 'very long-chain fatty acyl-CoA(4-)',
                          'definition': 'A fatty acyl-CoA(4-) arising from '
                                        'deprotonation of the phosphate and '
                                        'diphosphate OH groups of any very '
                                        'long-chain fatty acyl-CoA; major '
                                        'species at pH 7.3.',
                          'parents': ['CHEBI:77636']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.05660377358490566 is too low.\n'
               'True positives: '
               "[('CCCCCCCCCCCCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 32 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain')]\n"
               'False positives: '
               "[('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 26 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 32 "
               "carbons in chain'), "
               "('C[C@H](CCC[C@H](C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@H]1CC[C@H]2[C@@H]3[C@H](O)C[C@@H]4C[C@H](O)CC[C@]4(C)[C@H]3C[C@H](O)[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 36 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 34 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 36 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 26 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CC[C@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 32 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 36 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 36 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 34 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 26 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 34 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('[H][C@@](C)(CC[C@@H](O)[C@@H](C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@]1([H])CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])C[C@H](O)[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 26 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('C[C@H](CCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@H]1CC[C@H]2[C@@H]3CCC4=CC(=O)CC[C@]4(C)[C@H]3CC[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 38 "
               "carbons in chain'), "
               "('CC(=CCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@H]1CC[C@H]2[C@@H]3CC=C4C[C@@H](O)CC[C@]4(C)[C@H]3CC[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCC(O)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('C[C@H](CCC[C@H](C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@H]1CC[C@H]2[C@@H]3CCC4=CC(=O)CC[C@]4(C)[C@H]3CC[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 32 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 38 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 32 "
               "carbons in chain'), "
               "('C[C@H](C=CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@H]1CC[C@H]2[C@@H]3CCC4=CC(=O)CC[C@]4(C)[C@H]3CC[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('[H][C@@](C)(CCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@]1([H])CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])CC[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('[H][C@@](C)(CCC[C@@H](C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@]1([H])CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])CC[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 36 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 36 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 34 "
               "carbons in chain'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 32 "
               "carbons in chain'), "
               "('[H][C@@](C)(CCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@]1([H])CC[C@@]2([H])[C@]3([H])CC[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])C[C@H](O)[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 38 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 26 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 38 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 32 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 38 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 32 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 26 "
               "carbons in chain'), "
               "('C[C@@H]([C@H]1CC[C@H]2[C@@H]3CCC4=CC(=O)CC[C@]4(C)[C@H]3CC[C@]12C)C(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCC(O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 26 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 34 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 32 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 26 "
               "carbons in chain'), "
               "('[H][C@@](C)(CCC[C@@H](C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@]1([H])CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])C[C@H](O)[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 26 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 34 "
               "carbons in chain'), "
               "('C[C@H](CCC[C@H](C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@H]1CC[C@H]2[C@@H]3CC=C4C[C@@H](O)CC[C@]4(C)[C@H]3CC[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 36 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('[H][C@@](C)(CC\\\\C=C(/C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@]1([H])CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])C[C@H](O)[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain'), "
               "('[H][C@@](C)(CCC(=O)C(C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@]1([H])CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])C[C@H](O)[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 36 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 30 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 32 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCC\\\\C=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain'), "
               "('CCCCCCCCCCCCCCCCCCCCCC(O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCC(=O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCCCCCCCCCCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 34 "
               "carbons in chain'), "
               "('C[C@H](CCC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@H]1CC[C@H]2[C@@H]3CC=C4C[C@@H](O)CC[C@]4(C)[C@H]3CC[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 22 "
               "carbons in chain'), "
               "('[H][C@@](C)(CCCC(C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)[C@@]1([H])CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])C[C@H](O)[C@]12C', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 24 "
               "carbons in chain'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCCCCC[C@@H](O)CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Very long-chain fatty acyl-CoA(4-) with approximately 28 "
               "carbons in chain')]\n"
               'False negatives: []',
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 7,
    'num_true_negatives': 183897,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.2222222222222222,
    'recall': 0.6666666666666666,
    'f1': 0.3333333333333333,
    'accuracy': 0.9999564997525924}


"""
Classifies: CHEBI:131870 hydroxy monounsaturated fatty acid anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_hydroxy_monounsaturated_fatty_acid_anion(smiles: str):
    """
    Determines if a molecule is a hydroxy monounsaturated fatty acid anion.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a hydroxy monounsaturated fatty acid anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for carboxylate anion group
    carboxylate_pattern = Chem.MolFromSmarts("[C]([O-])=O")
    if not mol.HasSubstructMatch(carboxylate_pattern):
        return False, "No carboxylate anion group found"

    # Check for monounsaturation
    num_double_bonds = Descriptors.NumHDosityNodes(mol)
    if num_double_bonds != 1:
        return False, "Not monounsaturated"

    # Check for hydroxy group(s)
    hydroxy_pattern = Chem.MolFromSmarts("[O;H]")
    if not mol.HasSubstructMatch(hydroxy_pattern):
        return False, "No hydroxy group found"

    # Check for linear carbon chain
    carbon_chain = Chem.MolFromSmarts("[C;H3][C;H2][C;H2][C;H2][C;H2][C;H2][C;H2][C;H2][C;H2][C;H2][C;H2][C;H2][C;H2][C;H2][C;H2][C;H2]")
    if not mol.HasSubstructMatch(carbon_chain):
        return False, "Not a linear carbon chain"

    return True, "Molecule is a hydroxy monounsaturated fatty acid anion"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:131870',
                          'name': 'hydroxy monounsaturated fatty acid anion',
                          'definition': 'Any monounsaturated fatty acid anion '
                                        'carrying one or more hydroxy '
                                        'substituents.',
                          'parents': ['CHEBI:59835', 'CHEBI:82680']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.Descriptors' has no attribute "
             "'NumHDosityNodes'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:33641 olefin
"""
from rdkit import Chem
from rdkit.Chem import rdchem

def is_olefin(smiles: str):
    """
    Determines if a molecule is an olefin (acyclic or cyclic hydrocarbons with one or more carbon-carbon double bonds, excluding aromatic compounds).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is an olefin, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains only carbon and hydrogen atoms
    allowed_atoms = {'C', 'H'}
    if any(atom.GetSymbol() not in allowed_atoms for atom in mol.GetAtoms()):
        return False, "Molecule contains atoms other than carbon and hydrogen"

    # Check if the molecule contains any aromatic rings
    aromatic_rings = [ring for ring in mol.GetRingInfo().AtomRings() if all(mol.GetAtomWithIdx(idx).GetIsAromatic() for idx in ring)]
    if aromatic_rings:
        return False, "Molecule contains aromatic rings"

    # Check if the molecule contains any carbon-carbon double bonds
    double_bonds = [bond for bond in mol.GetBonds() if bond.GetBondType() == Chem.BondType.DOUBLE]
    if not double_bonds:
        return False, "Molecule does not contain any carbon-carbon double bonds"

    # Check if the double bonds are between carbon atoms
    for bond in double_bonds:
        atom1 = mol.GetAtomWithIdx(bond.GetBeginAtomIdx())
        atom2 = mol.GetAtomWithIdx(bond.GetEndAtomIdx())
        if atom1.GetSymbol() != 'C' or atom2.GetSymbol() != 'C':
            return False, "Double bond not between carbon atoms"

    return True, "Molecule is an olefin"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33641',
                          'name': 'olefin',
                          'definition': 'Acyclic and cyclic hydrocarbons '
                                        'having one or more carbon-carbon '
                                        'double bonds, apart from the formal '
                                        'ones in aromatic compounds. The class '
                                        'olefins subsumes alkenes and '
                                        'cycloalkenes and the corresponding '
                                        'polyenes.',
                          'parents': ['CHEBI:24632', 'CHEBI:78840']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: 'Mol' object has no attribute "
               "'GetAromaticRings'",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 30,
    'num_false_positives': 100,
    'num_true_negatives': 35489,
    'num_false_negatives': 8,
    'num_negatives': None,
    'precision': 0.23076923076923078,
    'recall': 0.7894736842105263,
    'f1': 0.35714285714285715,
    'accuracy': 0.9969685912369832}


"""
Classifies: CHEBI:19573 2-enoyl-CoA
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_2_enoyl_CoA(smiles: str):
    """
    Determines if a molecule is a 2-enoyl-CoA (unsaturated fatty acyl-CoA with double bond between positions 2-3).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a 2-enoyl-CoA, False otherwise
        str: Reason for classification
    """
    # Check for valid SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Look for CoA substructure pattern
    coA_pattern = Chem.MolFromSmarts('[#16]CCNC(=O)CCNC(=O)[CH]([OH])C(C)(C)COP([OH])(=O)OP([OH])(=O)OC[CH]1O[CH]([CH]([OH])[CH]1OP([OH])([OH])=O)n1cnc2c(N)ncnc12')
    
    # Look for thioester with alpha,beta unsaturation pattern
    thioester_pattern = Chem.MolFromSmarts('[#16]C(=O)C=C')

    if not mol.HasSubstructMatch(coA_pattern):
        return False, "Missing CoA structure"

    if not mol.HasSubstructMatch(thioester_pattern):
        return False, "Missing thioester with alpha,beta unsaturation"

    # Get the matches
    thioester_matches = mol.GetSubstructMatches(thioester_pattern)
    
    # For each thioester match, verify it's connected to the CoA part
    coA_matches = mol.GetSubstructMatches(coA_pattern)
    if coA_matches and thioester_matches:
        coA_sulfur = coA_matches[0][0]  # First atom (sulfur) in CoA pattern
        for match in thioester_matches:
            thioester_sulfur = match[0]  # First atom (sulfur) in thioester pattern
            if coA_sulfur == thioester_sulfur:
                return True, "Contains 2-enoyl-CoA structure with double bond between positions 2-3"

    return False, "Thioester and CoA parts not properly connected"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:19573',
                          'name': '2-enoyl-CoA',
                          'definition': 'An unsaturated fatty acyl-CoA in '
                                        'which the S-acyl group contains a '
                                        'double bond between positions 2 and '
                                        '3.',
                          'parents': ['CHEBI:51006']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               'False negatives: '
               "[('CC(C)(COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)C=C', "
               "'Missing 2-enoyl-CoA structure'), "
               "('CC(C)(COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\CC(O)=O', "
               "'Missing 2-enoyl-CoA structure'), "
               "('CC(C)(COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@@H](O)C(=O)NCCC(=O)NCCSC(=O)\\\\C=C\\\\C=C/[*]', "
               "'Missing 2-enoyl-CoA structure'), "
               "('CC(C)=CCC\\\\C(C)=C\\\\C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Missing 2-enoyl-CoA structure'), "
               "('C\\\\C(CC(O)=O)=C/C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)N1C=NC2=C1N=CN=C2N', "
               "'Missing 2-enoyl-CoA structure'), "
               "('S(CCNC(=O)CCNC(=O)[C@H](O)C(COP(OP(OC[C@H]1O[C@@H](N2C3=NC=NC(N)=C3N=C2)C(O)[C@H]1OP(O)(O)=O)(O)=O)(O)=O)(C)C)C(=O)\\\\C=C\\\\CCCCCCCCCCCCC', "
               "'Missing 2-enoyl-CoA structure')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 6,
    'num_false_positives': 100,
    'num_true_negatives': 138982,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.05660377358490566,
    'recall': 1.0,
    'f1': 0.10714285714285715,
    'accuracy': 0.9992810307143679}


"""
Classifies: CHEBI:16498 N-acylneuraminic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_N_acylneuraminic_acid(smiles: str):
    """
    Determines if a molecule is an N-acylneuraminic acid.

    N-acylneuraminic acids are defined as neuraminic acids carrying an N-acyl substituent.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an N-acylneuraminic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Identify the neuraminic acid core
    neuraminic_acid_core_smarts = "[C@](C([O-])=O)(O)[C@H](O)[C@H](O)[C@H](O)[C@H](NC(=O))"
    neuraminic_acid_core = mol.GetSubstructMatches(Chem.MolFromSmarts(neuraminic_acid_core_smarts))

    if not neuraminic_acid_core:
        return False, "Molecule does not contain the neuraminic acid core"

    # Check for an N-acyl substituent
    n_acyl_smarts = "[CH3][CX3](=[OX1])"
    n_acyl = mol.GetSubstructMatches(Chem.MolFromSmarts(n_acyl_smarts))

    if not n_acyl:
        return False, "Molecule does not contain an N-acyl substituent"

    # Check if the N-acyl substituent is attached to the nitrogen of the neuraminic acid core
    core_n_idx = neuraminic_acid_core[0][-1]
    for acyl_idx in n_acyl:
        acyl_atom = mol.GetAtomWithIdx(acyl_idx)
        if acyl_atom.GetNeighbors()[0].GetIdx() == core_n_idx:
            return True, "Molecule is an N-acylneuraminic acid"

    return False, "N-acyl substituent is not attached to the neuraminic acid core"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:16498',
                          'name': 'N-acylneuraminic acid',
                          'definition': 'Any neuraminic acid carrying an '
                                        'N-acyl substituent.',
                          'parents': ['CHEBI:26667']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183917,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945627942888}


"""
Classifies: CHEBI:22661 aspartic acid derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_aspartic_acid_derivative(smiles: str):
    """
    Determines if a molecule is an aspartic acid derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an aspartic acid derivative, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for basic carbon skeleton of aspartic acid
    # Looking for C-C-C with appropriate substitutions
    aspartic_pattern = Chem.MolFromSmarts('[#6]-[#6](-[#7,#15])-[#6]')
    if not mol.HasSubstructMatch(aspartic_pattern):
        return False, "Does not have basic aspartic acid carbon skeleton"
        
    # Check for two carboxylic acid groups (or derivatives)
    carboxyl_pattern = Chem.MolFromSmarts('C(=O)[OH,NH2,O-,N]')
    matches = mol.GetSubstructMatches(carboxyl_pattern)
    if len(matches) < 2:
        return False, "Does not have two carboxylic acid groups or derivatives"
        
    # Check for amino group or amino derivative
    amino_pattern = Chem.MolFromSmarts('[NH2,NH][#6,#15]')
    n_acyl_pattern = Chem.MolFromSmarts('[NH]C(=O)')
    
    if not (mol.HasSubstructMatch(amino_pattern) or mol.HasSubstructMatch(n_acyl_pattern)):
        return False, "Does not have amino group or derivative"
        
    # Check if it's a simple peptide (exclude these)
    peptide_pattern = Chem.MolFromSmarts('[NH]C(=O)[CH]([NH2,NH])C(=O)[NH]')
    if mol.HasSubstructMatch(peptide_pattern):
        return False, "Appears to be a peptide containing aspartic acid"
        
    # Additional checks for specific types of derivatives
    if mol.HasSubstructMatch(Chem.MolFromSmarts('[NH]C(=O)c1ccccc1')):
        return True, "N-acyl aspartic acid derivative"
    elif mol.HasSubstructMatch(Chem.MolFromSmarts('[CH]([NH2])[CH](O)')):
        return True, "Hydroxy-aspartic acid derivative"
    elif mol.HasSubstructMatch(Chem.MolFromSmarts('C(=O)[NH2]')):
        return True, "Aspartic acid amide derivative"
    elif mol.HasSubstructMatch(Chem.MolFromSmarts('[NH]C([*])=O')):
        return True, "N-substituted aspartic acid derivative"
    elif any(atom.GetIsotope() > 0 for atom in mol.GetAtoms()):
        return True, "Isotopically labeled aspartic acid derivative"
        
    return True, "Aspartic acid derivative"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22661',
                          'name': 'aspartic acid derivative',
                          'definition': 'An amino acid derivative resulting '
                                        'from reaction of aspartic acid at the '
                                        'amino group or either of the carboxy '
                                        'groups, or from the replacement of '
                                        'any hydrogen of aspartic acid by a '
                                        'heteroatom. The definition normally '
                                        'excludes peptides containing aspartic '
                                        'acid residues.',
                          'parents': ['CHEBI:83821']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 7,
    'num_false_positives': 100,
    'num_true_negatives': 319,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06542056074766354,
    'recall': 1.0,
    'f1': 0.12280701754385964,
    'accuracy': 0.7652582159624414}


"""
Classifies: CHEBI:24470 haloamino acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_haloamino_acid(smiles: str):
    """
    Determines if a molecule is a haloamino acid (non-proteinogenic amino acid with at least one halogen).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a haloamino acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Split salt forms if present
    fragments = Chem.GetMolFrags(mol, asMols=True)
    main_mol = max(fragments, key=lambda m: m.GetNumAtoms())
    
    # Check for amino acid pattern: has both NH2 and COOH groups
    amino_pattern = Chem.MolFromSmarts('[NX3H2,NX4H3+][CX4]')
    carboxyl_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    
    if not (main_mol.HasSubstructMatch(amino_pattern) and main_mol.HasSubstructMatch(carboxyl_pattern)):
        return False, "Missing amino acid backbone structure"

    # Check for halogens in the main molecule
    halogens = ['F', 'Cl', 'Br', 'I']
    found_halogens = []
    
    for atom in main_mol.GetAtoms():
        if atom.GetSymbol() in halogens and atom.GetIsotope() == 0:  # Exclude isotopes
            found_halogens.append(atom.GetSymbol())
            
    if not found_halogens:
        return False, "No halogen atoms found"

    # Check if it's connected to the main structure (not just a salt)
    for halogen in found_halogens:
        halogen_pattern = Chem.MolFromSmarts(f'[{halogen}]-[!Cl&!Br&!F&!I]')
        if main_mol.HasSubstructMatch(halogen_pattern):
            return True, f"Non-proteinogenic amino acid with halogen(s): {', '.join(set(found_halogens))}"
    
    return False, "Halogen present only as counterion"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24470',
                          'name': 'haloamino acid',
                          'definition': 'Any non-proteinogenic amino acid '
                                        'carrying at least one halo group.',
                          'parents': ['CHEBI:83820']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.0396039603960396 is too low.\n'
               "True positives: [('N[C@@H](CC1=CC=C(F)C=C1)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): F'), "
               "('N[C@@H](Cc1ccc(O)c(Br)c1)C(O)=O', 'Non-proteinogenic amino "
               "acid with halogen(s): Br')]\n"
               "False positives: [('NC(Cc1c[nH]c2c(Cl)cccc12)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('Cl.NC(CC1C(=O)NC2=CC=CC=C12)C(O)=O', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('N[C@H](Cc1ccc(O)c(F)c1)C(O)=O', 'Non-proteinogenic amino "
               "acid with halogen(s): F'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSC1C=CC(Br)=CC1O)C(=O)NCC(O)=O)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Br'), "
               "('N[C@@H](CS\\\\C(Cl)=C\\\\Cl)C(O)=O', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('C1=C(C=C(C(=C1O)O)Cl)CC(C(=O)O)N', 'Non-proteinogenic amino "
               "acid with halogen(s): Cl'), "
               "('ClC1=CC=2NC=C(C2C=C1)C[C@@H]3NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@H](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)C[C@H](N)C(=O)O)C(O)C4=CC=C(O)C=C4)CO)CCC(=O)O)C(O)C)COC(=O)[C@H]5N(C([C@@H](NC([C@H](NC3=O)C(C)C)=O)CC6=CC=CC=C6)=O)CC(C)C5O)C(C(=O)N)C', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('IC=1C=C(C[C@@H](N)C(O)=O)C=CC1OC2=CC=C(O)C=C2', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('Cl.NCCCC[C@H](N)C(O)=O', 'Non-proteinogenic amino acid with "
               "halogen(s): Cl'), ('[I-].[S+](CC[C@H](N)C(O)=O)(C)C', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('N[C@@H](CN1C=C(F)C(=O)NC1O)C(O)=O', 'Non-proteinogenic amino "
               "acid with halogen(s): F'), "
               "('N[C@@H](Cc1cc(O)c(O)cc1[18F])C(O)=O', 'Non-proteinogenic "
               "amino acid with halogen(s): F'), "
               "('FC1=CC=2NC=C(CC(N)C(O)=O)C2C=C1', 'Non-proteinogenic amino "
               "acid with halogen(s): F'), "
               "('ClCCSC[C@H](NC(=O)CC[C@H](N)C(O)=O)C(=O)NCC(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('FC=1C(C(N)C(O)=O)=CC=CC1', 'Non-proteinogenic amino acid "
               "with halogen(s): F'), "
               "('C1(=CNC2=C1C=C(C=C2)Cl)C[C@@H](C(=O)O)N', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('C1=CC(=CC=C1CC(C(=O)O)N)Cl', 'Non-proteinogenic amino acid "
               "with halogen(s): Cl'), "
               "('C1=CC(=CC=C1NC(=O)CC(C(=O)O)N)OC2=CC(=C(C=C2Br)F)F', "
               "'Non-proteinogenic amino acid with halogen(s): F, Br'), "
               "('IC1=C(O[C@@H]2O[C@@H]([C@@H](O)[C@H](O)[C@H]2O)C(O)=O)C=CC(OC3=C(I)C=C(C[C@H](N)C(O)=O)C=C3I)=C1', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('C1(=CNC2=C1C=CC(=C2)Cl)CC(C(=O)O)N', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('ClC=1C=2C(CC(N)C(O)=O)=CNC2C=CC1', 'Non-proteinogenic amino "
               "acid with halogen(s): Cl'), "
               "('N[C@@H](CS\\\\C(Cl)=C/Cl)C(O)=O', 'Non-proteinogenic amino "
               "acid with halogen(s): Cl'), "
               "('N[C@@H](Cc1cc(I)c(Oc2ccc(OS(O)(=O)=O)c(I)c2)c(I)c1)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('Cl.Cl.CC(=N)NCCCC[C@H](N)C(O)=O', 'Non-proteinogenic amino "
               "acid with halogen(s): Cl'), ('ClC(Cl)C[C@H](N)C(=O)O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('N[C@H](C(O)=O)CC1=CC(Cl)=C(C(Cl)=C1)OCC2=C3OC(C4=CC=CC=C4)=NC3=CC(N)=C2.Cl.Cl', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('N[C@@H](Cc1ccc(cc1)N(CCCl)CCCl)C(O)=O', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('P(N(CCCl)CCCl)(N(CCCl)CCCl)(OCCS(C[C@@H](C(N[C@@H](C(O)=O)C1=CC=CC=C1)=O)NC(CC[C@@H](C(O)=O)N)=O)(=O)=O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('Cl.NCCC[C@@H](N)C(O)=O', 'Non-proteinogenic amino acid with "
               "halogen(s): Cl'), "
               "('N[C@@H](CCCNC(N)=N)C(O)=O.OC(=O)C[C@H]1CCC2=C1NC1=C2C=C(OCC2=CC(=C(C=C2)C2CCCC2)C(F)(F)F)C=C1', "
               "'Non-proteinogenic amino acid with halogen(s): F'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSC(O)C(Cl)Cl)C(=O)NCC(O)=O)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSCCl)C(=O)NCC(=O)O)C(=O)O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSC1C(O)C=CC=C1Br)C(=O)NCC(O)=O)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Br'), "
               "('C1(=CNC2=C1C=CC(=C2Cl)Cl)C[C@@H](C(=O)O)N', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('N[C@@H](Cn1cc(F)c(=O)[nH]c1=O)C(O)=O', 'Non-proteinogenic "
               "amino acid with halogen(s): F'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSCCN(CCCl)c1ccc(CC(N)C(O)=O)cc1)C(=O)NCC(O)=O)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('NC(Cc1ccc(O)c(Br)c1)C(O)=O', 'Non-proteinogenic amino acid "
               "with halogen(s): Br'), ('Cl.OC(=O)[C@@H](N)CCCN', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('ClC(=C)CC[C@H](N)C(=O)O', 'Non-proteinogenic amino acid with "
               "halogen(s): Cl'), ('C(C(C(O)=O)N)C1=CC=CC=C1F', "
               "'Non-proteinogenic amino acid with halogen(s): F'), "
               "('C([C@@H](C(O)=O)N)C1=CC=CC=C1F', 'Non-proteinogenic amino "
               "acid with halogen(s): F'), "
               "('IC=1C=C(OC2=CC=C(C[C@@H](N)C(O)=O)C=C2)C=CC1OS(O)(=O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('N[C@@H](Cc1c[nH]c2c(Cl)cccc12)C(O)=O', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('C1=CC(=CC=C1O)OC2=C(C=C(C=C2I)C[C@@H](C(=O)O)N)I', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('C([C@H](CS)N)(=O)O.Cl', 'Non-proteinogenic amino acid with "
               "halogen(s): Cl'), ('[H][C@]1(CC(Cl)=NO1)[C@H](N)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('ClC1=NO[C@@H]([C@@H]1O)C(N)C(=O)O', 'Non-proteinogenic amino "
               "acid with halogen(s): Cl'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSC(=O)CCl)C(=O)NCC(O)=O)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('CC(CC(N)C(O)=O)C(F)(F)F', 'Non-proteinogenic amino acid with "
               "halogen(s): F'), "
               "('IC1=C(OC2OC(C(O)C(O)C2O)C(O)=O)C(I)=CC(OC3=C(I)C=C(CC(N)C(O)=O)C=C3I)=C1', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('IC=1C=CC(C[C@H](C(O)=O)N)=CC1', 'Non-proteinogenic amino "
               "acid with halogen(s): I'), ('[Cl-].C[S+](C)CC[C@H](N)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('IC1=CC(CC(N)C(O)=O)=CC(I)=C1OC2=CC=C(O)C=C2', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('ClC1=C(O)C(Cl)=CC(=C1)C(=O)NC(C(=O)O)CC2NC(C(=O)O)(C(O)C(N)C(=O)O)CC2', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('C1(=CNC2=C1C=CC(=C2)Cl)C[C@H](C(=O)O)N', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('NC(NCCC[C@@H](C(O)=O)N)=N.Cl', 'Non-proteinogenic amino acid "
               "with halogen(s): Cl'), "
               "('C1=CC(=C(C=C1OC2=C(C=C(C=C2I)CC(C(=O)O)N)I)I)O', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('IC1=C(O[C@@H]2O[C@@H]([C@H](O)[C@H](O)[C@@H]2O)C(O)=O)C(I)=CC(OC3=CC=C(C[C@@H](N)C(O)=O)C=C3)=C1', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('C(=O)([C@@H](N)CC(CCN)Cl)O', 'Non-proteinogenic amino acid "
               "with halogen(s): Cl'), ('N[C@@H](Cc1cccc(F)c1)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): F'), "
               "('ClC1=CC(N2N=C(C=C2)C)=C([C@@H](OC3=NC(=NC(C4=CC=C(C[C@H](N)C(O)=O)C=C4)=C3)N)C(F)(F)F)C=C1', "
               "'Non-proteinogenic amino acid with halogen(s): F, Cl'), "
               "('NCCCC[C@H](N)C(O)=O.Cc1c(Cl)cccc1Nc1ncccc1C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('IC=1C=CC(C[C@@H](C(O)=O)N)=CC1', 'Non-proteinogenic amino "
               "acid with halogen(s): I'), "
               "('N(C(=O)C=1C(=NOC1C)C2=C(C=CC=C2F)Cl)[C@@H]([C@@]3(N[C@H](C(S3)(C)C)C(O)=O)[H])C(=O)NCCCC[C@@H](C(=O)O)N', "
               "'Non-proteinogenic amino acid with halogen(s): F, Cl'), "
               "('N[C@@H](CCSC(F)(F)F)C(O)=O', 'Non-proteinogenic amino acid "
               "with halogen(s): F'), ('Cl.NCCCC[C@@H](N)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('N[C@@H](Cc1ccc(OC(=O)c2c[nH]c3cc(Br)c(O)cc23)cc1)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Br'), "
               "('N[C@@H](CCC(=O)N[C@@H](CS\\\\C(Cl)=C/Cl)C(=O)NCC(O)=O)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('NC(Cc1c[nH]c2ccc(F)cc12)C(O)=O', 'Non-proteinogenic amino "
               "acid with halogen(s): F'), ('N[C@@H](Cc1cc(O)c(O)cc1F)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): F'), "
               "('NC(Cc1cc(Br)c(O)c(Br)c1)C(O)=O', 'Non-proteinogenic amino "
               "acid with halogen(s): Br'), ('NC(CF)CCC(N)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): F'), "
               "('NC(Cc1ccc(cc1)N(CCCl)CCCl)C(O)=O', 'Non-proteinogenic amino "
               "acid with halogen(s): Cl'), ('C([C@H](CS)N)(=O)O.Cl.O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSCC(Cl)=O)C(=O)NCC(O)=O)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('Cl/C=C\\\\C[C@H](N)C(=O)O', 'Non-proteinogenic amino acid "
               "with halogen(s): Cl'), "
               "('IC1=C(OC2OC(C(O)C(O)C2O)C(O)=O)C=CC(OC3=C(I)C=C(CC(N)C(O)=O)C=C3I)=C1', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('ClC=1C(C[C@H](N)C(O)=O)=CC=CC1', 'Non-proteinogenic amino "
               "acid with halogen(s): Cl'), ('C([C@H](C(O)=O)N)C1=CC=CC=C1F', "
               "'Non-proteinogenic amino acid with halogen(s): F'), "
               "('C1=C(C(=CC(=C1O)O)Cl)CC(C(=O)O)N', 'Non-proteinogenic amino "
               "acid with halogen(s): Cl'), ('N[C@@H](CC(Cl)=C)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('IC=1C=CC(CC(C(O)=O)N)=CC1', 'Non-proteinogenic amino acid "
               "with halogen(s): I'), ('N[C@@H](CCCCNC(=O)C(F)(F)F)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): F'), "
               "('ClCCN(C1=CC=C(C[C@H](N)C(O)=O)C=C1)CCO', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('C1(=CNC2=C1C=CC(=C2)Cl)C[C@@H](C(=O)O)N', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('Cl.O=C(O)[C@@H](N)C1(CC1)C', 'Non-proteinogenic amino acid "
               "with halogen(s): Cl'), ('BrC1=CC=C(CC(N)C(O)=O)C=C1', "
               "'Non-proteinogenic amino acid with halogen(s): Br'), "
               "('N[C@@H](CCC(=O)N[C@@H](CSC(C(=O)[O-])Cl)C(=O)NCC(O)=O)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('ClC1=C(O)C=CC(=C1)[C@@H](N2C(=O)[C@@H](NC(=O)/C(=N/O)/C3=CC=C(OCC[C@@H](N)C(=O)O)C=C3)C2)C(=O)O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('N[C@@H](Cc1ccc(Oc2ccc(OS(O)(=O)=O)c(I)c2)c(I)c1)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('N[C@H](Cc1c[nH]c2c(Cl)cccc12)C(O)=O', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('IC1=C(OC2O[C@@H]([C@@H](O)[C@H](O)[C@H]2O)C(O)=O)C(I)=CC(OC3=C(I)C=C(C[C@H](N)C(O)=O)C=C3I)=C1', "
               "'Non-proteinogenic amino acid with halogen(s): I'), "
               "('[H]C(Cl)=C(Cl)SC[C@H](N)C(O)=O', 'Non-proteinogenic amino "
               "acid with halogen(s): Cl'), "
               "('NC(CCC(=O)NC(CSC1CCOP(=O)(N1)N(CCCl)CCCl)C(=O)NCC(O)=O)C(O)=O', "
               "'Non-proteinogenic amino acid with halogen(s): Cl'), "
               "('Cl.Cl.S(/C(=N\\\\CCC[C@H](N)C(O)=O)/N)C', 'Non-proteinogenic "
               "amino acid with halogen(s): Cl'), "
               "('C1=CC(=C(C(=C1CC(C(=O)O)N)Cl)O)O', 'Non-proteinogenic amino "
               "acid with halogen(s): Cl')]\n"
               "False negatives: [('N[14C@]1(C[C@H](F)C1)C(O)=O', 'Missing "
               "amino acid backbone structure')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 72971,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9986315242083368}


"""
Classifies: CHEBI:139589 retinoid anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_retinoid_anion(smiles: str):
    """
    Determines if a molecule is a retinoid anion.
    
    Args:
        smiles (str): SMILES string of the molecule
    
    Returns:
        bool: True if molecule is a retinoid anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for carboxylate anion
    carboxylate_pattern = Chem.MolFromSmarts('C(=O)[O-]')
    if not mol.HasSubstructMatch(carboxylate_pattern):
        return False, "No carboxylate anion group found"

    # Pattern for retinoid core with flexible chain length and terminal carboxylate
    # This pattern allows for variations in the cyclohexene ring substitution
    retinoid_pattern = Chem.MolFromSmarts('[#6]1~[#6]~[#6]~[#6]~[#6]~[#6]1~[#6]=[#6]~[#6]=[#6]~[#6]=[#6]~[#6]=C-C(=O)[O-]')
    
    # Pattern for retinoid glucuronate derivatives
    retinoid_glucuronate = Chem.MolFromSmarts('[#6]1~[#6]~[#6]~[#6]~[#6]~[#6]1~[#6]=[#6]~[#6]=[#6]~[#6]=[#6]~[#6]-C(=O)O[C]1[C][C]([C]([C]([C]1)C(=O)[O-])O)O')

    if mol.HasSubstructMatch(retinoid_pattern):
        # Additional check for conjugated system
        conjugated_system = Chem.MolFromSmarts('C=CC=CC=CC=C')
        if mol.HasSubstructMatch(conjugated_system):
            return True, "Retinoid anion with conjugated system and carboxylate group"
    
    if mol.HasSubstructMatch(retinoid_glucuronate):
        return True, "Retinoid glucuronate derivative with carboxylate group"

    # Check for specific pattern with methyl substitutions
    specific_pattern = Chem.MolFromSmarts('[CH3]-[#6]1-[#6](-[CH3])-[#6]-[#6]-[#6](-[CH3])-[#6]1-[#6]=[#6]-[#6](-[CH3])=[#6]-[#6]=[#6]-[#6](-[CH3])=[#6]-C(=O)[O-]')
    if mol.HasSubstructMatch(specific_pattern):
        return True, "Methylated retinoid anion"

    return False, "Not a retinoid anion structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:139589',
                          'name': 'retinoid anion',
                          'definition': 'A carboxylic acid anion obtained by '
                                        'deprotonation of any retinoid carboxy '
                                        'group.',
                          'parents': ['CHEBI:29067']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               'False negatives: '
               "[('C1(C)(CO)CCC(C(=C1\\\\C=C\\\\C(=C\\\\C=C\\\\C(=C\\\\C(=O)[O-])\\\\C)\\\\C)C)O', "
               "'No retinoid core structure found'), "
               "('C1C(C(=C(CC1)C)/C=C/C(/C)=C/C=C/C(/C)=C/C(O[C@H]2[C@@H]([C@H]([C@@H]([C@H](O2)C(=O)[O-])O)O)O)=O)(C)C', "
               "'No retinoid core structure found')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 6,
    'num_true_negatives': 183908,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.14285714285714285,
    'recall': 0.5,
    'f1': 0.22222222222222224,
    'accuracy': 0.999961939146132}


"""
Classifies: CHEBI:26469 quaternary nitrogen compound
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_quaternary_nitrogen_compound(smiles: str):
    """
    Determines if a molecule contains a quaternary nitrogen (N+) that is electronically neutral.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains quaternary nitrogen, False otherwise
        str: Reason for classification
    """
    try:
        mol = Chem.MolFromSmiles(smiles)
        if mol is None:
            return False, "Invalid SMILES string"
            
        # Look for quaternary nitrogen atoms (N+)
        quaternary_nitrogens = []
        for atom in mol.GetAtoms():
            if atom.GetSymbol() == 'N' and atom.GetFormalCharge() == 1:
                # Check if nitrogen has 4 bonds
                if len(atom.GetBonds()) == 4:
                    quaternary_nitrogens.append(atom)
                    
        if not quaternary_nitrogens:
            return False, "No quaternary nitrogen atoms found"
            
        # Check if molecule is electronically neutral overall
        total_charge = sum(atom.GetFormalCharge() for atom in mol.GetAtoms())
        if total_charge != 0:
            return False, "Molecule is not electronically neutral"
            
        # Get substituents on quaternary nitrogens
        substituents = []
        for n in quaternary_nitrogens:
            for neighbor in n.GetNeighbors():
                substituents.append(neighbor.GetSymbol())
                
        return True, f"Contains quaternary nitrogen with substituents: {', '.join(set(substituents))}"
        
    except Exception as e:
        return None, f"Error processing molecule: {str(e)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26469',
                          'name': 'quaternary nitrogen compound',
                          'definition': 'A nitrogen molecular entity that is '
                                        'electronically neutral but which '
                                        'contains a quaternary nitrogen.',
                          'parents': ['CHEBI:35352']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 200,
    'num_false_positives': 100,
    'num_true_negatives': 26059,
    'num_false_negatives': 8,
    'num_negatives': None,
    'precision': 0.6666666666666666,
    'recall': 0.9615384615384616,
    'f1': 0.7874015748031495,
    'accuracy': 0.9959039708726818}


"""
Classifies: CHEBI:147335 1-phosphatidyl-1D-myo-inositol-3-phosphate anion derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_1_phosphatidyl_1D_myo_inositol_3_phosphate_anion_derivative(smiles: str):
    """
    Determines if a molecule is a 1-phosphatidyl-1D-myo-inositol-3-phosphate anion derivative.

    A 1-phosphatidyl-1D-myo-inositol-3-phosphate anion derivative is defined as a phosphatidylinositol
    anion where the inositol is phosphorylated at position 3 and either phosphorylated or not at
    position 4 and/or position 5; major species at pH 7.3.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a 1-phosphatidyl-1D-myo-inositol-3-phosphate anion derivative, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find the inositol ring
    inositol_ring = None
    for ring in mol.GetRingInfo().AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetSymbol() == 'O' for atom in atoms):
                inositol_ring = ring
                break

    if inositol_ring is None:
        return False, "No inositol ring found"

    # Check if inositol is phosphorylated at position 3
    inositol_atoms = [mol.GetAtomWithIdx(i) for i in inositol_ring]
    phosphate_count = sum(1 for atom in inositol_atoms if atom.GetDegree() == 4)
    if phosphate_count < 1:
        return False, "Inositol ring is not phosphorylated at position 3"

    # Check if inositol is phosphorylated or not at position 4 and/or position 5
    phosphate_count = sum(1 for atom in inositol_atoms if atom.GetDegree() == 4)
    if phosphate_count < 1 or phosphate_count > 3:
        return False, "Inositol ring is not phosphorylated at positions 3, 4, and/or 5"

    # Check if the molecule is an anion
    formal_charge = Chem.GetFormalCharge(mol)
    if formal_charge >= 0:
        return False, "Molecule is not an anion"

    return True, "Molecule is a 1-phosphatidyl-1D-myo-inositol-3-phosphate anion derivative"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:147335',
                          'name': '1-phosphatidyl-1D-myo-inositol-3-phosphate '
                                  'anion derivative',
                          'definition': 'A phosphatidylinositol anion where '
                                        'the inositol is phosphorylated at '
                                        'position 3 and either phosphorylated '
                                        'or not at position 4 and/or position '
                                        '5; major species at pH 7.3.',
                          'parents': ['CHEBI:62643']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183924,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945630012234}


"""
Classifies: CHEBI:25562 nitrophenol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_nitrophenol(smiles: str):
    """
    Determines if a molecule is a nitrophenol (phenol with at least one nitro group).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a nitrophenol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for phenol substructure
    phenol_pattern = Chem.MolFromSmarts('c1ccccc1[OH]')
    if not mol.HasSubstructMatch(phenol_pattern):
        return False, "No phenol substructure found"

    # Check for nitro group
    nitro_pattern = Chem.MolFromSmarts('[N+](=O)[O-]')
    nitro_matches = mol.GetSubstructMatches(nitro_pattern)
    
    if not nitro_matches:
        return False, "No nitro groups found"

    # Count nitro groups
    num_nitro = len(nitro_matches)

    # Get aromatic ring atoms
    phenol_match = mol.GetSubstructMatch(phenol_pattern)
    ring_atoms = set(phenol_match[:6])  # First 6 atoms are the aromatic ring

    # Check if any nitro group is attached to the phenol ring
    nitro_on_ring = False
    for nitro_match in nitro_matches:
        nitro_n = nitro_match[0]  # Index of nitrogen atom
        for neighbor in mol.GetAtomWithIdx(nitro_n).GetNeighbors():
            if neighbor.GetIdx() in ring_atoms:
                nitro_on_ring = True
                break
        if nitro_on_ring:
            break

    if not nitro_on_ring:
        return False, "Nitro group(s) not attached to phenol ring"

    # Get positions of substituents
    positions = []
    for i, atom_idx in enumerate(phenol_match[:6]):
        atom = mol.GetAtomWithIdx(atom_idx)
        for neighbor in atom.GetNeighbors():
            if neighbor.GetIdx() not in phenol_match:
                positions.append(i+1)
                break

    return True, f"Nitrophenol with {num_nitro} nitro group(s) at position(s) {sorted(set(positions))}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25562',
                          'name': 'nitrophenol',
                          'definition': 'Any member of the class of  phenols '
                                        'or substituted phenols carrying  at '
                                        'least 1 nitro group.',
                          'parents': ['CHEBI:33853', 'CHEBI:35716']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 6,
    'num_false_positives': 100,
    'num_true_negatives': 180937,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.05660377358490566,
    'recall': 0.8571428571428571,
    'f1': 0.10619469026548672,
    'accuracy': 0.9994421245664038}


"""
Classifies: CHEBI:26932 tetrapyrrole
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_tetrapyrrole(smiles: str):
    """
    Determines if a molecule is a tetrapyrrole, defined as a natural pigment containing four pyrrole rings
    joined by one-carbon units linking position 2 of one pyrrole ring to position 5 of the next.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a tetrapyrrole, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find all rings
    rings = mol.GetRingInfo().AtomRings()

    # Check for four pyrrole rings
    pyrrole_rings = []
    for ring in rings:
        atoms = [mol.GetAtomWithIdx(i) for i in ring]
        if len(ring) == 4:
            num_n = sum(atom.GetAtomicNum() == 7 for atom in atoms)
            if num_n == 4:  # All atoms in the ring are nitrogen
                pyrrole_rings.append(ring)

    if len(pyrrole_rings) != 4:
        return False, "Not exactly four pyrrole rings found"

    # Check if the pyrrole rings are connected by one-carbon units
    connected_rings = set()
    for i, ring1 in enumerate(pyrrole_rings):
        for j, ring2 in enumerate(pyrrole_rings):
            if i != j:
                for atom1_idx in ring1:
                    atom1 = mol.GetAtomWithIdx(atom1_idx)
                    for atom2_idx in ring2:
                        atom2 = mol.GetAtomWithIdx(atom2_idx)
                        if atom1.GetIdx() in atom2.GetNeighbors():
                            connected_rings.add((i, j))

    if len(connected_rings) != 4:
        return False, "Pyrrole rings are not correctly connected by one-carbon units"

    return True, "Molecule is a tetrapyrrole"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26932',
                          'name': 'tetrapyrrole',
                          'definition': 'A natural pigment containing four '
                                        'pyrrole rings joined by one-carbon '
                                        'units linking position 2 of one '
                                        'pyrrole ring to position 5 of the '
                                        'next.',
                          'parents': ['CHEBI:33833', 'CHEBI:38077']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183622,
    'num_false_negatives': 31,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9998312034107801}


"""
Classifies: CHEBI:26420 pyridinemonocarboxylic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_pyridinemonocarboxylic_acid(smiles: str):
    """
    Determines if a molecule is a pyridine monocarboxylic acid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a pyridine monocarboxylic acid, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Find pyridine rings
    pyridine_rings = []
    ring_info = mol.GetRingInfo()
    
    for ring in ring_info.AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            # Check if ring is aromatic and contains exactly one nitrogen
            if (all(atom.GetIsAromatic() for atom in atoms) and
                sum(1 for atom in atoms if atom.GetSymbol() == 'N') == 1):
                pyridine_rings.append(ring)
                
    if not pyridine_rings:
        return False, "No pyridine ring found"

    # Find carboxylic acid groups (-C(=O)OH)
    carboxyl_pattern = Chem.MolFromSmarts('[CX3](=O)[OX2H1]')
    carboxyl_matches = mol.GetSubstructMatches(carboxyl_pattern)
    
    if not carboxyl_matches:
        return False, "No carboxylic acid group found"
        
    if len(carboxyl_matches) > 1:
        return False, "More than one carboxylic acid group found"

    # Check if carboxylic acid is attached to pyridine ring
    carboxyl_carbon = carboxyl_matches[0][0]
    carboxyl_atom = mol.GetAtomWithIdx(carboxyl_carbon)
    
    for pyridine_ring in pyridine_rings:
        ring_atoms = set(pyridine_ring)
        # Check neighbors of carboxyl carbon
        for neighbor in carboxyl_atom.GetNeighbors():
            if neighbor.GetIdx() in ring_atoms:
                position = get_pyridine_position(mol, pyridine_ring, neighbor.GetIdx())
                return True, f"Pyridine-{position}-carboxylic acid"
                
    return False, "Carboxylic acid group not directly attached to pyridine ring"

def get_pyridine_position(mol, ring_atoms, carbon_idx):
    """Helper function to determine position of substituent on pyridine ring"""
    # Find nitrogen atom in ring
    n_idx = None
    for idx in ring_atoms:
        if mol.GetAtomWithIdx(idx).GetSymbol() == 'N':
            n_idx = idx
            break
            
    # Get path around ring from N to substituted C
    path = Chem.GetShortestPath(mol, n_idx, carbon_idx)
    if len(path) == 1:
        return "2"  # Ortho
    elif len(path) == 2:
        return "3"  # Meta  
    else:
        return "4"  # Para


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26420',
                          'name': 'pyridinemonocarboxylic acid',
                          'definition': 'A monocarboxylic acid in which the '
                                        'carboxy group is attached to a '
                                        'pyridine (or substituted pyridine) '
                                        'ring.',
                          'parents': [   'CHEBI:25384',
                                         'CHEBI:26421',
                                         'CHEBI:33859']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 67113,
    'num_false_negatives': 7,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 0.3,
    'f1': 0.05309734513274337,
    'accuracy': 0.9984082828793717}